//------------------------------------------------------------------------------
//
// File:    apsconfig.h
//
// Purpose: Config parameters
//   
// (c) 2004 Plekhanov Andrey
//
// Initial version 0.1 06.03.2004
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//------------------------------------------------------------------------------

#ifndef 	_APSCONFIG_H_
#define         _APSCONFIG_H_	1

enum {
  PARAM_INTEGER = 0,
  PARAM_DOUBLE,
  PARAM_STRING
};

class APSIStrFile;

//------------------------------- APSAbsParameter ------------------------------

class APSAbsParameter
{
  private:

    char * Name;

  public:

    APSAbsParameter( const char * aName );

    virtual ~APSAbsParameter( void );

    const char * GetName( void ) const
      { return( Name ); }
};

//------------------------------- APSParameter ---------------------------------

class APSParameter : public APSAbsParameter
{
  private:

    int                  Type;
    const APSParameter * pNextParameter;

  public:

    APSParameter( const char * aName, int aType, const APSParameter * apNextParameter );

    virtual ~APSParameter( void );

    int GetType( void ) const
      { return( Type ); }

    const APSParameter * GetNextParameterPtr( void ) const
      { return( pNextParameter ); }
};

//------------------------------- APSIntegerParameter -------------------------

class APSIntegerParameter : public APSAbsParameter
{
  private:

    int                         Value;
    const APSIntegerParameter * pNextParameter;

  public:

    APSIntegerParameter( const char * aName, int aValue, const APSIntegerParameter * apNextParameter );

    virtual ~APSIntegerParameter( void );

    int GetValue( void ) const
      { return( Value ); }

    const APSIntegerParameter * GetNextParameterPtr( void ) const
      { return( pNextParameter ); }
};

//------------------------------- APSDoubleParameter -------------------------

class APSDoubleParameter : public APSAbsParameter
{
  private:

    double                     Value;
    const APSDoubleParameter * pNextParameter;

  public:

    APSDoubleParameter( const char * aName, double aValue, const APSDoubleParameter * apNextParameter );

    virtual ~APSDoubleParameter( void );

    double GetValue( void ) const
      { return( Value ); }

    const APSDoubleParameter * GetNextParameterPtr( void ) const
      { return( pNextParameter ); }
};

//------------------------------- APSStringParameter -------------------------

class APSStringParameter : public APSAbsParameter
{
  private:

    char                     * Value;
    const APSStringParameter * pNextParameter;

  public:

    APSStringParameter( const char * aName, const char * aValue, const APSStringParameter * apNextParameter );

    virtual ~APSStringParameter( void );

    const char * GetValue( void ) const
      { return( Value ); }

    const APSStringParameter * GetNextParameterPtr( void ) const
      { return( pNextParameter ); }
};

//------------------------------- APSConfig ---------------------------------

class APSConfig
{
  private:

    APSParameter        * pFirstParameter;

    APSIntegerParameter * pIntegerFirstParameter;

    APSDoubleParameter  * pDoubleFirstParameter;

    APSStringParameter  * pStringFirstParameter;  

    void AddIntegerParameter( const char * Name, int Value );

    void AddDoubleParameter( const char * Name, double Value );

    void AddStringParameter( const char * Name, const char * Value );

    const APSIntegerParameter * FindIntegerParameter( const char * Name ) const;

    const APSDoubleParameter * FindDoubleParameter( const char * Name ) const;

    const APSStringParameter * FindStringParameter( const char * Name ) const;

    int IfExistsParameter( const char * Name ) const;

  protected:

    friend class APSReadConfig;

    int AddParameter( const char * Name, int Type );  

    const APSParameter * FindParameter( const char * Name ) const;

  public:

    APSConfig( void );

    virtual ~APSConfig( void );

    int GetIntegerValue( const char * Name, int * Value ) const;

    int GetDoubleValue( const char * Name, double * Value ) const;

    int GetStringValue( const char * Name, char * Value ) const;
};

#endif
