//------------------------------------------------------------------------------
//
// File:    apsfile.h
//
// Purpose: General purpose procedures for file processing
//   
// (c) 2004 Plekhanov Andrey
//
// Initial version 0.1 30.04.2004
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//------------------------------------------------------------------------------

#ifndef 	_APSFILE_H_
#define         _APSFILE_H_	1

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>

class APSFile
{
  private:

    FILE * pHandler;
    int    opened;
    char * FileName;
    char * Mode;

  protected:

    FILE * GetHandlerPtr( void ) const;

    int GetDescriptor( void ) const;

  public:

    APSFile( const char * aFileName, const char * aMode );

    virtual ~APSFile( void );

    const char * GetFileName( void ) const;

    int IfOpened( void ) const;

    int Open( void );

    int Close( void );

    off_t LSeek( off_t Offset, int Whence ) const;

    int IfExists( void ) const;

    /* device file resides on */
    int GetDevice( dev_t * p_dev )      const;

    /* the file serial number */
    ino_t     GetSerialNumber( void )	const;

    /* file mode */
    mode_t    GetMode( void )		const;

    /* number of hard links to the file */
    int GetLinkNumber( nlink_t * p_nlink ) const;

    /* user ID of owner */
    int GetUserID( uid_t * p_uid )      const;

    /* group ID of owner */
    int GetGroupID( gid_t * p_gid )     const;

    /* the device identifier (special files only)*/
    int GetDevID( dev_t * p_dev )       const;

    /* total size of file, in bytes */
    off_t     GetLength( void )		const;

    /* file last access time */
    time_t    GetLastAccess( void )	const;

    /* file last modify time */
    time_t    GetLastModify( void )	const;

    /* file last status change time */
    time_t    GetLastStatus( void )	const;

    /* preferred blocksize for file system I/O*/
    long      GetBlockSize( void )	const;

    /* actual number of blocks allocated */
    long      GetBlockNumber( void )	const;

    /* Make an exclusive lock */
    int Lock( void ) const;

    /* Unlock file */
    int UnLock( void ) const;
};

#endif
