//------------------------------------------------------------------------------
//
// File:    apsmodule.cc
//
// Purpose: Module support
//   
// (c) 2004 Plekhanov Andrey
//
// Initial version 0.1 30.04.2004
// Check ( apOwner == 0 ) 16.12.2004
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//------------------------------------------------------------------------------

#include <stdio.h>

#include "newstr.h"
#include "apsmodule.h"
#include "apssubmodule.h"

const int MAX_MSG_LENGTH = 1024;

APSModule :: APSModule( const char * apModuleName, APSSubModule * apOwner ) : APSAbsModule()
{
  pOwner = apOwner;
  pModuleName = newStr( apModuleName );

  if( apOwner ) {
    apOwner->SetSubModule( this );
  }
}

APSModule :: ~APSModule( void )
{
  if( pOwner ) {
    pOwner->SetSubModule( 0 );
  }

  delete [] pModuleName;
}

const char * APSModule :: GetModuleName( void ) const
{
  return( pModuleName );
}

int APSModule :: InfoMessage( int MsgNumber, const char * pMsg, const APSAbsModule * pSubModule ) const
{
  static char MsgText[ MAX_MSG_LENGTH ];
  int         RetCode;

  RetCode = MODULE_CONTINUE;

  if( pOwner ) {
    if( pSubModule ) {
      RetCode = pOwner->InfoMessage( MsgNumber, pMsg, pSubModule );
    }
    else {
      RetCode = pOwner->InfoMessage( MsgNumber, pMsg, this );
    }
  }
  else {
    if( pSubModule ) {
      if( pMsg ) {
        snprintf( MsgText, MAX_MSG_LENGTH - 1, "INFO: %s: %s %s", pSubModule->GetModuleName(), pSubModule->GetMessageText( MsgNumber ), pMsg );
        PutMessage( MsgText );
      }
      else {
        snprintf( MsgText, MAX_MSG_LENGTH - 1, "INFO: %s: %s", pSubModule->GetModuleName(), pSubModule->GetMessageText( MsgNumber ) );
        PutMessage( MsgText );
      }
    }
    else {
      if( pMsg ) {
        snprintf( MsgText, MAX_MSG_LENGTH - 1, "INFO: %s: %s %s", GetModuleName(), GetMessageText( MsgNumber ), pMsg );
        PutMessage( MsgText );
      }
      else {
        snprintf( MsgText, MAX_MSG_LENGTH - 1, "INFO: %s: %s", GetModuleName(), GetMessageText( MsgNumber ) );
        PutMessage( MsgText );
      }
    }
  }

  return( RetCode );
}

int APSModule :: WarningMessage( int MsgNumber, const char * pMsg, const APSAbsModule * pSubModule ) const
{
  static char MsgText[ MAX_MSG_LENGTH ];
  int         RetCode;

  RetCode = MODULE_CONTINUE;

  if( pOwner ) {
    if( pSubModule ) {
      RetCode = pOwner->WarningMessage( MsgNumber, pMsg, pSubModule );
    }
    else {
      RetCode = pOwner->WarningMessage( MsgNumber, pMsg, this );
    }
  }
  else {
    if( pSubModule ) {
      if( pMsg ) {
        snprintf( MsgText, MAX_MSG_LENGTH - 1, "WARNING: %s: %s %s", pSubModule->GetModuleName(), pSubModule->GetMessageText( MsgNumber ), pMsg );
        PutMessage( MsgText );
      }
      else {
        snprintf( MsgText, MAX_MSG_LENGTH - 1, "WARNING: %s: %s", pSubModule->GetModuleName(), pSubModule->GetMessageText( MsgNumber ) );
        PutMessage( MsgText );
      }
    }
    else {
      if( pMsg ) {
        snprintf( MsgText, MAX_MSG_LENGTH - 1, "WARNING: %s: %s %s", GetModuleName(), GetMessageText( MsgNumber ), pMsg );
        PutMessage( MsgText );
      }
      else {
        snprintf( MsgText, MAX_MSG_LENGTH - 1, "WARNING: %s: %s", GetModuleName(), GetMessageText( MsgNumber ) );
        PutMessage( MsgText );
      }
    }
  }

  return( RetCode );
}

int APSModule :: ErrorMessage( int MsgNumber, const char * pMsg, const APSAbsModule * pSubModule ) const
{
  static char MsgText[ MAX_MSG_LENGTH ];
  int         RetCode;

  RetCode = MODULE_STOP;

  if( pOwner ) {
    if( pSubModule ) {
      RetCode = pOwner->ErrorMessage( MsgNumber, pMsg, pSubModule );
    }
    else {
      RetCode = pOwner->ErrorMessage( MsgNumber, pMsg, this );
    }
  }
  else {
    if( pSubModule ) {
      if( pMsg ) {
        snprintf( MsgText, MAX_MSG_LENGTH - 1, "ERROR: %s: %s %s", pSubModule->GetModuleName(), pSubModule->GetMessageText( MsgNumber ), pMsg );
        PutMessage( MsgText );
      }
      else {
        snprintf( MsgText, MAX_MSG_LENGTH - 1, "ERROR: %s: %s", pSubModule->GetModuleName(), pSubModule->GetMessageText( MsgNumber ) );
        PutMessage( MsgText );
      }
    }
    else {
      if( pMsg ) {
        snprintf( MsgText, MAX_MSG_LENGTH - 1, "ERROR: %s: %s %s", GetModuleName(), GetMessageText( MsgNumber ), pMsg );
        PutMessage( MsgText );
      }
      else {
        snprintf( MsgText, MAX_MSG_LENGTH - 1, "ERROR: %s: %s", GetModuleName(), GetMessageText( MsgNumber ) );
        PutMessage( MsgText );
      }
    }
  }

  return( RetCode );
}

int APSModule :: DebugMessage( int MsgNumber, const char * pMsg, const APSAbsModule * pSubModule ) const
{
  static char MsgText[ MAX_MSG_LENGTH ];
  int         RetCode;

  RetCode = MODULE_CONTINUE;

  if( pOwner ) {
    if( pSubModule ) {
      RetCode = pOwner->DebugMessage( MsgNumber, pMsg, pSubModule );
    }
    else {
      RetCode = pOwner->DebugMessage( MsgNumber, pMsg, this );
    }
  }
  else {
    if( pSubModule ) {
      if( pMsg ) {
        snprintf( MsgText, MAX_MSG_LENGTH - 1, "DEBUG: %s: %s %s", pSubModule->GetModuleName(), pSubModule->GetMessageText( MsgNumber ), pMsg );
        PutMessage( MsgText );
      }
      else {
        snprintf( MsgText, MAX_MSG_LENGTH - 1, "DEBUG: %s: %s", pSubModule->GetModuleName(), pSubModule->GetMessageText( MsgNumber ) );
        PutMessage( MsgText );  
      }
    }
    else {
      if( pMsg ) {
        snprintf( MsgText, MAX_MSG_LENGTH - 1, "DEBUG: %s: %s %s", GetModuleName(), GetMessageText( MsgNumber ), pMsg );
        PutMessage( MsgText );
      }
      else {
        snprintf( MsgText, MAX_MSG_LENGTH - 1, "DEBUG: %s: %s", GetModuleName(), GetMessageText( MsgNumber ) );
        PutMessage( MsgText );
      }
    }
  }

  return( RetCode );
}

int APSModule :: StrMessage( int MsgNumber, const char * pMsg, const APSAbsModule * pSubModule ) const
{
  static char MsgText[ MAX_MSG_LENGTH ];
  int         RetCode;

  RetCode = MODULE_CONTINUE;

  if( pOwner ) {
    if( pSubModule ) {
      RetCode = pOwner->StrMessage( MsgNumber, pMsg, pSubModule );
    }
    else {
      RetCode = pOwner->StrMessage( MsgNumber, pMsg, this );
    }
  }
  else {
    if( pSubModule ) {
      if( pMsg ) {
        snprintf( MsgText, MAX_MSG_LENGTH - 1, "%s %s", pSubModule->GetMessageText( MsgNumber ), pMsg );
        PutMessage( MsgText );
      }
      else {
        snprintf( MsgText, MAX_MSG_LENGTH - 1, "%s", pSubModule->GetMessageText( MsgNumber ) );
        PutMessage( MsgText );
      }
    }
    else {
      if( pMsg ) {
        snprintf( MsgText, MAX_MSG_LENGTH - 1, "%s %s", GetMessageText( MsgNumber ), pMsg );
        PutMessage( MsgText );
      }
      else {
        snprintf( MsgText, MAX_MSG_LENGTH - 1, "%s", GetMessageText( MsgNumber ) );
        PutMessage( MsgText );
      }
    }
  }

  return( RetCode );
}

const char * APSModule :: GetMessageText( int MsgNumber ) const
{
  static char MsgText[ MAX_MSG_LENGTH ];

  sprintf( MsgText, "%d\n", MsgNumber );

  return( MsgText );
}

void APSModule :: PutMessage( const char * pMsg ) const
{
  printf("%s", pMsg );
  fflush( stdout );
}
